import _Promise from 'babel-runtime/core-js/promise';
import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import { some, map, keyBy, forEach } from 'lodash';

// https://medium.com/@mweststrate/how-to-safely-use-react-context-b7e343eff076

var Fields = function () {
  function Fields() {
    var _this = this;

    _classCallCheck(this, Fields);

    this.getFieldsOptions = function () {
      return keyBy(map(_this.getFields(), function (field) {
        return _this.getFieldOptions(field.name);
      }), 'name');
    };

    this.getValues = function () {
      var values = {};
      forEach(_this.getFieldsOptions(), function (field) {
        values[field.name] = field.value;
      });
      return values;
    };

    this.getErrors = function () {
      var errors = {};
      forEach(_this.getFieldsOptions(), function (field) {
        errors[field.name] = field.errors;
      });
      return errors;
    };

    this.clear();
  }

  Fields.prototype.getFields = function getFields() {
    return this.fields;
  };

  Fields.prototype.getFieldByName = function getFieldByName(name) {
    return this.getFields()[name];
  };

  Fields.prototype.getFieldOptions = function getFieldOptions(name) {
    var field = this.getFieldByName(name);
    if (field) {
      return field.getFieldOptions();
    }
  };

  Fields.prototype.updateField = function updateField(field) {
    this.fields[field.name] = field;
  };

  Fields.prototype.isValid = function isValid() {
    var fields = this.getFieldsOptions();
    var isValid = true;
    forEach(fields, function (field) {
      if (field.pending) {
        isValid = false;
      }
      if (field.errors.length || field.asyncErrors.length) {
        isValid = false;
      }
    });
    return isValid;
  };

  Fields.prototype.updateFieldsAsUsed = function updateFieldsAsUsed() {
    forEach(this.getFields(), function (field) {
      field.instance.validate();
    });
  };

  Fields.prototype.reset = function reset() {
    var value = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    var promises = [];
    forEach(this.getFields(), function (field, key) {
      promises.push(field.instance.reset(value[key]));
    });
    return _Promise.all(promises);
  };

  Fields.prototype.removeField = function removeField(name) {
    delete this.fields[name];
  };

  Fields.prototype.subscribe = function subscribe(f, name) {
    var subscriber = {
      callback: f,
      id: ++this.lastId
    };
    if (this.subscribers[name]) {
      this.subscribers[name].push(subscriber);
    } else {
      this.subscribers[name] = [subscriber];
    }
    this.publish(name);
    return this.lastId;
  };

  Fields.prototype.publish = function publish(name) {
    var _this2 = this;

    if (this.subscribers[name]) {
      this.subscribers[name].forEach(function (subscriber) {
        if (name) {
          subscriber.callback(_this2.getFieldOptions(name));
        } else {
          subscriber.callback(_this2.getFieldsOptions());
        }
      });
    }
  };

  Fields.prototype.removeSubscriber = function removeSubscriber(id) {
    if (!id) {
      return null;
    }
    forEach(this.subscribers, function (subscribers) {
      for (var i = subscribers.length - 1; i >= 0; i--) {
        if (subscribers[i].id === id) {
          subscribers.splice(i, 1);
          return true;
        }
      }
    });
  };

  Fields.prototype.clear = function clear() {
    this.subscribers = {};
    this.fields = {};
    this.lastId = 0;
  };

  return Fields;
}();

var Validators = function () {
  function Validators() {
    _classCallCheck(this, Validators);

    this.clear();
  }

  Validators.prototype.updateFields = function updateFields() {
    var fields = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    this.fields = fields;
  };

  Validators.prototype.updateValidators = function updateValidators(validators) {
    this.validators = validators;
    this.errors = this.getErrors();
  };

  Validators.prototype.getErrors = function getErrors() {
    var _this3 = this;

    var errors = {};
    forEach(this.validators, function (validator, name) {
      errors[name] = validator(_this3.fields);
    });
    this.errors = errors;
    return errors;
  };

  Validators.prototype.isValid = function isValid() {
    var errors = this.getErrors();
    return !some(errors, function (v) {
      return !!v;
    });
  };

  Validators.prototype.subscribe = function subscribe(f, name) {
    var subscriber = {
      callback: f,
      id: ++this.lastId
    };
    var sName = name || '__all__';
    if (this.subscribers[sName]) {
      this.subscribers[sName].push(subscriber);
    } else {
      this.subscribers[sName] = [subscriber];
    }
    return this.lastId;
  };

  Validators.prototype.publish = function publish() {
    var _this4 = this;

    var errors = this.getErrors();
    forEach(errors, function (error, name) {
      if (name !== '__all__') {
        var _subscribers = _this4.subscribers[name] || [];
        _subscribers.forEach(function (subscriber) {
          subscriber.callback(error);
        });
      }
    });

    var subscribers = this.subscribers['__all__'] || [];
    subscribers.forEach(function (subscriber) {
      subscriber.callback(errors);
    });
  };

  Validators.prototype.removeSubscriber = function removeSubscriber(id) {
    if (!id) {
      return null;
    }
    forEach(this.subscribers, function (subscribers) {
      for (var i = subscribers.length - 1; i >= 0; i--) {
        if (subscribers[i].id === id) {
          subscribers.splice(i, 1);
          return true;
        }
      }
    });
  };

  Validators.prototype.clear = function clear() {
    this.subscribers = {};
    this.validators = {};
    this.fields = {};
    this.errors = {};
    this.lastId = 0;
  };

  return Validators;
}();

export var FormContext = function () {
  function FormContext() {
    _classCallCheck(this, FormContext);

    this.initialized = false;
    this.fields = new Fields();
    this.validators = new Validators();
    this.errors = [];
    this.formValidators = {};
    this.formValidatorsOptions = {};
    this.formAsyncValidator = {};
    this.formAsyncValidatorOptions = {};
  }

  FormContext.prototype.update = function update() {
    this.validators.updateFields(this.fields.getFieldsOptions());
    this.validators.updateValidators(this.formValidators['']);
    this.validators.publish();
  };

  FormContext.prototype.onInitField = function onInitField(field) {
    // console.log('onInitField', field);
    this.fields.updateField(field);
    this.fields.publish(field.name);
    this.fields.publish('');
    if (this.initialized) {
      this.update();
    }
  };

  FormContext.prototype.onUpdateField = function onUpdateField(field) {
    // console.log('onUpdateField', field);
    this.fields.updateField(field);
    this.fields.publish(field.name);
    this.fields.publish('');
    this.update();
  };

  FormContext.prototype.onRemoveField = function onRemoveField(field) {
    // console.log('onRemoveField', field);
    this.fields.removeField(field.name);
    this.update();
  };

  FormContext.prototype.setValidators = function setValidators() {
    var validators = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    this.formValidators = validators;
  };

  FormContext.prototype.setValidatorsOptions = function setValidatorsOptions() {
    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    this.formValidatorsOptions = options;
  };

  FormContext.prototype.setAsyncValidator = function setAsyncValidator() {
    var validators = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    this.formAsyncValidator = validators;
  };

  FormContext.prototype.setAsyncValidatorOptions = function setAsyncValidatorOptions() {
    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    this.formAsyncValidatorOptions = options;
  };

  FormContext.prototype.isValid = function isValid() {
    return this.validators.isValid() && this.fields.isValid();
  };

  FormContext.prototype.reset = function reset(value) {
    var _this5 = this;

    this.initialized = false;
    this.fields.reset(value).then(function () {
      _this5.initialized = true;
      _this5.update();
    });
  };

  FormContext.prototype.clear = function clear() {
    this.setValidators();
    this.setValidatorsOptions();
    this.setAsyncValidator();
    this.setAsyncValidatorOptions();
    this.fields.clear();
    this.validators.clear();
  };

  return FormContext;
}();